@@ -1,4 +1,5 @@
 #!/usr/bin/env python3
+"""Extract application metadata from a source repository."""
 #
 # import_subcommand.py - part of the FDroid server tools
 # Copyright (C) 2010-13, Ciaran Gultnieks, ciaran@ciarang.com
@@ -17,33 +18,63 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import json
+import logging
 import os
 import re
+import shutil
 import stat
+import sys
 import urllib
+from argparse import ArgumentParser
+from pathlib import Path
+from typing import Optional
 
 import git
-import json
-import shutil
-import sys
 import yaml
-from argparse import ArgumentParser
-import logging
-from pathlib import Path
 
 try:
     from yaml import CSafeLoader as SafeLoader
 except ImportError:
     from yaml import SafeLoader
 
-from . import _
-from . import common
-from . import metadata
+from . import _, common, metadata
 from .exception import FDroidException
 
-
 config = None
-options = None
+
+SETTINGS_GRADLE_REGEX = re.compile(r'settings\.gradle(?:\.kts)?')
+GRADLE_SUBPROJECT_REGEX = re.compile(r'''['"]:?([^'"]+)['"]''')
+APPLICATION_ID_REGEX = re.compile(r'''\s*applicationId\s=?\s?['"].*['"]''')
+
+
+def get_all_gradle_and_manifests(build_dir):
+    paths = []
+    for root, dirs, files in os.walk(build_dir):
+        for f in sorted(files):
+            if f == 'AndroidManifest.xml' or f.endswith(('.gradle', '.gradle.kts')):
+                full = Path(root) / f
+                paths.append(full)
+    return paths
+
+
+def get_gradle_subdir(build_dir, paths):
+    """Get the subdir where the gradle build is based."""
+    first_gradle_dir = None
+    for path in paths:
+        if not first_gradle_dir:
+            first_gradle_dir = path.parent.relative_to(build_dir)
+        if path.exists() and SETTINGS_GRADLE_REGEX.match(path.name):
+            for m in GRADLE_SUBPROJECT_REGEX.finditer(path.read_text(encoding='utf-8')):
+                for f in (path.parent / m.group(1)).glob('build.gradle*'):
+                    with f.open(encoding='utf-8') as fp:
+                        for line in fp:
+                            if common.ANDROID_PLUGIN_REGEX.match(
+                                line
+                            ) or APPLICATION_ID_REGEX.match(line):
+                                return f.parent.relative_to(build_dir)
+    if first_gradle_dir and first_gradle_dir != Path('.'):
+        return first_gradle_dir
 
 
 def handle_retree_error_on_windows(function, path, excinfo):
@@ -53,7 +84,20 @@ def handle_retree_error_on_windows(function, path, excinfo):
         function(path)
 
 
-def clone_to_tmp_dir(app):
+def clone_to_tmp_dir(app: metadata.App, rev=None) -> Path:
+    """Clone the source repository of an app to a temporary directory for further processing.
+
+    Parameters
+    ----------
+    app
+        The App instance to clone the source of.
+
+    Returns
+    -------
+    tmp_dir
+        The (temporary) directory the apps source has been cloned into.
+
+    """
     tmp_dir = Path('tmp')
     tmp_dir.mkdir(exist_ok=True)
 
@@ -62,12 +106,12 @@ def clone_to_tmp_dir(app):
     if tmp_dir.exists():
         shutil.rmtree(str(tmp_dir), onerror=handle_retree_error_on_windows)
     vcs = common.getvcs(app.RepoType, app.Repo, tmp_dir)
-    vcs.gotorevision(options.rev)
+    vcs.gotorevision(rev)
 
     return tmp_dir
 
 
-def getrepofrompage(url):
+def getrepofrompage(url: str) -> tuple[Optional[str], str]:
     """Get the repo type and address from the given web page.
 
     The page is scanned in a rather naive manner for 'git clone xxxx',
@@ -75,6 +119,18 @@ def getrepofrompage(url):
     that's the information we want.  Returns repotype, address, or
     None, reason
 
+    Parameters
+    ----------
+    url
+        The url to look for repository information at.
+
+    Returns
+    -------
+    repotype_or_none
+        The found repository type or None if an error occured.
+    address_or_reason
+        The address to the found repository or the reason if an error occured.
+
     """
     if not url.startswith('http'):
         return (None, _('{url} does not start with "http"!'.format(url=url)))
@@ -97,7 +153,7 @@ def getrepofrompage(url):
     index = page.find('hg clone')
     if index != -1:
         repotype = 'hg'
-        repo = page[index + 9:]
+        repo = page[index + 9 :]
         index = repo.find('<')
         if index == -1:
             return (None, _("Error while getting repo address"))
@@ -109,7 +165,7 @@ def getrepofrompage(url):
     index = page.find('git clone')
     if index != -1:
         repotype = 'git'
-        repo = page[index + 10:]
+        repo = page[index + 10 :]
         index = repo.find('<')
         if index == -1:
             return (None, _("Error while getting repo address"))
@@ -120,13 +176,30 @@ def getrepofrompage(url):
     return (None, _("No information found.") + page)
 
 
-def get_app_from_url(url):
+def get_app_from_url(url: str) -> metadata.App:
     """Guess basic app metadata from the URL.
 
     The URL must include a network hostname, unless it is an lp:,
     file:, or git/ssh URL.  This throws ValueError on bad URLs to
     match urlparse().
 
+    Parameters
+    ----------
+    url
+        The URL to look to look for app metadata at.
+
+    Returns
+    -------
+    app
+        App instance with the found metadata.
+
+    Raises
+    ------
+    :exc:`~fdroidserver.exception.FDroidException`
+        If the VCS type could not be determined.
+    :exc:`ValueError`
+        If the URL is invalid.
+
     """
     parsed = urllib.parse.urlparse(url)
     invalid_url = False
@@ -183,29 +256,53 @@ def get_app_from_url(url):
 
 
 def main():
+    """Extract app metadata and write it to a file.
 
-    global config, options
+    The behaviour of this function is influenced by the configuration file as
+    well as command line parameters.
+
+    Raises
+    ------
+    :exc:`~fdroidserver.exception.FDroidException`
+        If the repository already has local metadata, no URL is specified and
+        the current directory is not a Git repository, no application ID could
+        be found, no Gradle project could be found or there is already metadata
+        for the found application ID.
+
+    """
+    global config
 
     # Parse command line...
     parser = ArgumentParser()
     common.setup_global_opts(parser)
-    parser.add_argument("-u", "--url", default=None,
-                        help=_("Project URL to import from."))
-    parser.add_argument("-s", "--subdir", default=None,
-                        help=_("Path to main Android project subdirectory, if not in root."))
-    parser.add_argument("-c", "--categories", default=None,
-                        help=_("Comma separated list of categories."))
-    parser.add_argument("-l", "--license", default=None,
-                        help=_("Overall license of the project."))
-    parser.add_argument("--omit-disable", action="store_true", default=False,
-                        help=_("Do not add 'disable:' to the generated build entries"))
-    parser.add_argument("--rev", default=None,
-                        help=_("Allows a different revision (or git branch) to be specified for the initial import"))
+    parser.add_argument("-u", "--url", help=_("Project URL to import from."))
+    parser.add_argument(
+        "-s",
+        "--subdir",
+        help=_("Path to main Android project subdirectory, if not in root."),
+    )
+    parser.add_argument(
+        "-c",
+        "--categories",
+        help=_("Comma separated list of categories."),
+    )
+    parser.add_argument("-l", "--license", help=_("Overall license of the project."))
+    parser.add_argument(
+        "--omit-disable",
+        action="store_true",
+        help=_("Do not add 'disable:' to the generated build entries"),
+    )
+    parser.add_argument(
+        "--rev",
+        help=_(
+            "Allows a different revision (or git branch) to be specified for the initial import"
+        ),
+    )
     metadata.add_metadata_arguments(parser)
-    options = parser.parse_args()
+    options = common.parse_args(parser)
     metadata.warnings_action = options.W
 
-    config = common.read_config(options)
+    config = common.read_config()
 
     apps = metadata.read_metadata()
     app = None
@@ -214,43 +311,42 @@ def main():
 
     local_metadata_files = common.get_local_metadata_files()
     if local_metadata_files:
-        raise FDroidException(_("This repo already has local metadata: %s") % local_metadata_files[0])
+        raise FDroidException(
+            _("This repo already has local metadata: %s") % local_metadata_files[0]
+        )
 
     build = metadata.Build()
+    app = metadata.App()
     if options.url is None and Path('.git').is_dir():
-        app = metadata.App()
-        app.AutoName = Path.cwd().name
         app.RepoType = 'git'
-
-        if Path('build.gradle').exists() or Path('build.gradle.kts').exists():
-            build.gradle = ['yes']
-
-        git_repo = git.Repo(Path.cwd())
+        tmp_importer_dir = Path.cwd()
+        git_repo = git.Repo(tmp_importer_dir)
         for remote in git.Remote.iter_items(git_repo):
             if remote.name == 'origin':
                 url = git_repo.remotes.origin.url
-                if url.startswith('https://git'):  # github, gitlab
-                    app.SourceCode = url.rstrip('.git')
-                app.Repo = url
+                app = get_app_from_url(url)
                 break
         write_local_file = True
     elif options.url:
         app = get_app_from_url(options.url)
-        tmp_importer_dir = clone_to_tmp_dir(app)
+        tmp_importer_dir = clone_to_tmp_dir(app, options.rev)
         git_repo = git.Repo(tmp_importer_dir)
 
         if not options.omit_disable:
-            build.disable = 'Generated by import.py - check/set version fields and commit id'
+            build.disable = (
+                'Generated by `fdroid import` - check version fields and commitid'
+            )
         write_local_file = False
     else:
         raise FDroidException("Specify project url.")
 
+    app.AutoUpdateMode = 'Version'
     app.UpdateCheckMode = 'Tags'
     build.commit = common.get_head_commit_id(git_repo)
 
     # Extract some information...
-    paths = common.get_all_gradle_and_manifests(tmp_importer_dir)
-    subdir = common.get_gradle_subdir(tmp_importer_dir, paths)
+    paths = get_all_gradle_and_manifests(tmp_importer_dir)
+    gradle_subdir = get_gradle_subdir(tmp_importer_dir, paths)
     if paths:
         versionName, versionCode, appid = common.parse_androidmanifests(paths, app)
         if not appid:
@@ -268,16 +364,15 @@ def main():
 
     # Create a build line...
     build.versionName = versionName or 'Unknown'
+    app.CurrentVersion = build.versionName
     build.versionCode = versionCode or 0
+    app.CurrentVersionCode = build.versionCode
     if options.subdir:
         build.subdir = options.subdir
-        build.gradle = ['yes']
-    elif subdir:
-        build.subdir = subdir.as_posix()
-        build.gradle = ['yes']
-    else:
-        # subdir might be None
-        subdir = Path()
+    elif gradle_subdir:
+        build.subdir = gradle_subdir.as_posix()
+    # subdir might be None
+    subdir = Path(tmp_importer_dir / build.subdir) if build.subdir else tmp_importer_dir
 
     if options.license:
         app.License = options.license
@@ -285,38 +380,54 @@ def main():
         app.Categories = options.categories.split(',')
     if (subdir / 'jni').exists():
         build.buildjni = ['yes']
-    if (subdir / 'build.gradle').exists() or (subdir / 'build.gradle').exists():
+    if (subdir / 'build.gradle').exists() or (subdir / 'build.gradle.kts').exists():
         build.gradle = ['yes']
 
+    app.AutoName = common.fetch_real_name(subdir, build.gradle)
+
     package_json = tmp_importer_dir / 'package.json'  # react-native
     pubspec_yaml = tmp_importer_dir / 'pubspec.yaml'  # flutter
     if package_json.exists():
-        build.sudo = ['apt-get update || apt-get update', 'apt-get install npm', 'npm install -g react-native-cli']
-        build.init = ['npm install']
+        build.sudo = [
+            'sysctl fs.inotify.max_user_watches=524288 || true',
+            'apt-get update',
+            'apt-get install -y npm',
+        ]
+        build.init = ['npm install --build-from-source']
         with package_json.open() as fp:
             data = json.load(fp)
-        app.AutoName = data.get('name', app.AutoName)
+        app.AutoName = app.AutoName or data.get('name')
         app.License = data.get('license', app.License)
         app.Description = data.get('description', app.Description)
         app.WebSite = data.get('homepage', app.WebSite)
         app_json = tmp_importer_dir / 'app.json'
+        build.scanignore = ['android/build.gradle']
+        build.scandelete = ['node_modules']
         if app_json.exists():
             with app_json.open() as fp:
                 data = json.load(fp)
-            app.AutoName = data.get('name', app.AutoName)
+            app.AutoName = app.AutoName or data.get('name')
     if pubspec_yaml.exists():
         with pubspec_yaml.open() as fp:
             data = yaml.load(fp, Loader=SafeLoader)
-        app.AutoName = data.get('name', app.AutoName)
+        app.AutoName = app.AutoName or data.get('name')
         app.License = data.get('license', app.License)
         app.Description = data.get('description', app.Description)
+        app.UpdateCheckData = 'pubspec.yaml|version:\\s.+\\+(\\d+)|.|version:\\s(.+)\\+'
         build.srclibs = ['flutter@stable']
-        build.output = 'build/app/outputs/apk/release/app-release-unsigned.apk'
+        build.output = 'build/app/outputs/flutter-apk/app-release.apk'
         build.subdir = None
         build.gradle = None
-        build.build = [
+        build.prebuild = [
+            'export PUB_CACHE=$(pwd)/.pub-cache',
             '$$flutter$$/bin/flutter config --no-analytics',
             '$$flutter$$/bin/flutter packages pub get',
+        ]
+        build.scandelete = [
+            '.pub-cache',
+        ]
+        build.build = [
+            'export PUB_CACHE=$(pwd)/.pub-cache',
             '$$flutter$$/bin/flutter build apk',
         ]
 
@@ -324,7 +435,7 @@ def main():
     if git_modules.exists():
         build.submodules = True
 
-    metadata.post_metadata_parse(app)
+    metadata.post_parse_yaml_metadata(app)
 
     app['Builds'].append(build)
 
@@ -335,8 +446,11 @@ def main():
         Path('build').mkdir(exist_ok=True)
         build_dir = Path('build') / appid
         if build_dir.exists():
-            logging.warning(_('{path} already exists, ignoring import results!')
-                            .format(path=build_dir))
+            logging.warning(
+                _('{path} already exists, ignoring import results!').format(
+                    path=build_dir
+                )
+            )
             sys.exit(1)
         elif tmp_importer_dir:
             # For Windows: Close the repo or a git.exe instance holds handles to repo
