@@ -20,20 +20,28 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+"""Process the index files.
+
+This module is loaded by all fdroid subcommands since it is loaded in
+fdroidserver/__init__.py.  Any narrowly used dependencies should be
+imported where they are used to limit dependencies for subcommands
+like publish/signindex/gpgsign.  This eliminates the need to have
+these installed on the signing server.
+
+"""
+
 import collections
 import hashlib
 import json
 import logging
 import os
 import re
-import ruamel.yaml
 import shutil
 import sys
 import tempfile
 import urllib.parse
 import zipfile
 import calendar
-import qrcode
 from binascii import hexlify, unhexlify
 from datetime import datetime, timezone
 from pathlib import Path
@@ -42,9 +50,9 @@ from xml.dom.minidom import Document
 from . import _
 from . import common
 from . import metadata
-from . import net
 from . import signindex
-from fdroidserver.common import ANTIFEATURES_CONFIG_NAME, CATEGORIES_CONFIG_NAME, CONFIG_CONFIG_NAME, MIRRORS_CONFIG_NAME, RELEASECHANNELS_CONFIG_NAME, DEFAULT_LOCALE, FDroidPopen, FDroidPopenBytes, load_stats_fdroid_signing_key_fingerprints
+from fdroidserver.common import ANTIFEATURES_CONFIG_NAME, CATEGORIES_CONFIG_NAME, CONFIG_CONFIG_NAME, MIRRORS_CONFIG_NAME, RELEASECHANNELS_CONFIG_NAME, DEFAULT_LOCALE, FDroidPopen, FDroidPopenBytes, load_publish_signer_fingerprints
+from fdroidserver._yaml import yaml
 from fdroidserver.exception import FDroidException, VerificationException
 
 
@@ -117,14 +125,13 @@ def make(apps, apks, repodir, archive):
                 raise TypeError(_('only accepts strings, lists, and tuples'))
         requestsdict[command] = packageNames
 
-    fdroid_signing_key_fingerprints = load_stats_fdroid_signing_key_fingerprints()
+    signer_fingerprints = load_publish_signer_fingerprints()
 
-    make_v0(sortedapps, apks, repodir, repodict, requestsdict,
-            fdroid_signing_key_fingerprints)
-    make_v1(sortedapps, apks, repodir, repodict, requestsdict,
-            fdroid_signing_key_fingerprints)
-    make_v2(sortedapps, apks, repodir, repodict, requestsdict,
-            fdroid_signing_key_fingerprints, archive)
+    make_v0(sortedapps, apks, repodir, repodict, requestsdict, signer_fingerprints)
+    make_v1(sortedapps, apks, repodir, repodict, requestsdict, signer_fingerprints)
+    make_v2(
+        sortedapps, apks, repodir, repodict, requestsdict, signer_fingerprints, archive
+    )
     make_website(sortedapps, repodir, repodict)
     make_altstore(
         sortedapps,
@@ -160,6 +167,7 @@ def make_website(apps, repodir, repodict):
     html_file = os.path.join(repodir, html_name)
 
     if _should_file_be_generated(html_file, autogenerate_comment):
+        import qrcode
         qrcode.make(link_fingerprinted).save(os.path.join(repodir, "index.png"))
         with open(html_file, 'w') as f:
             name = repodict["name"]
@@ -700,7 +708,7 @@ def v2_repo(repodict, repodir, archive):
     return repo
 
 
-def make_v2(apps, packages, repodir, repodict, requestsdict, fdroid_signing_key_fingerprints, archive):
+def make_v2(apps, packages, repodir, repodict, requestsdict, signer_fingerprints, archive):
 
     def _index_encoder_default(obj):
         if isinstance(obj, set):
@@ -722,7 +730,7 @@ def make_v2(apps, packages, repodir, repodict, requestsdict, fdroid_signing_key_
         output["repo"]["requests"] = requestsdict
 
     # establish sort order of the index
-    v1_sort_packages(packages, fdroid_signing_key_fingerprints)
+    sort_package_versions(packages, signer_fingerprints)
 
     output_packages = collections.OrderedDict()
     output['packages'] = output_packages
@@ -837,7 +845,7 @@ def make_v2(apps, packages, repodir, repodict, requestsdict, fdroid_signing_key_
         signindex.sign_index(repodir, json_name)
 
 
-def make_v1(apps, packages, repodir, repodict, requestsdict, fdroid_signing_key_fingerprints):
+def make_v1(apps, packages, repodir, repodict, requestsdict, signer_fingerprints):
 
     def _index_encoder_default(obj):
         if isinstance(obj, set):
@@ -867,7 +875,7 @@ def make_v1(apps, packages, repodir, repodict, requestsdict, fdroid_signing_key_
         output['repo']['mirrors'] = mirrors
 
     # establish sort order of the index
-    v1_sort_packages(packages, fdroid_signing_key_fingerprints)
+    sort_package_versions(packages, signer_fingerprints)
 
     appslist = []
     output['apps'] = appslist
@@ -975,8 +983,8 @@ def _copy_to_local_copy_dir(repodir, f):
                               .format(path=local_copy_dir))
 
 
-def v1_sort_packages(packages, fdroid_signing_key_fingerprints):
-    """Sort the supplied list to ensure a deterministic sort order for package entries in the index file.
+def sort_package_versions(packages, signer_fingerprints):
+    """Sort to ensure a deterministic order for package versions in the index file.
 
     This sort-order also expresses
     installation preference to the clients.
@@ -1001,7 +1009,7 @@ def v1_sort_packages(packages, fdroid_signing_key_fingerprints):
         if dev_signer and dev_signer == signer:
             group = GROUP_DEV_SIGNED
         else:
-            fdroid_signer = fdroid_signing_key_fingerprints.get(packageName, {}).get('signer')
+            fdroid_signer = signer_fingerprints.get(packageName, {}).get('signer')
             if fdroid_signer and fdroid_signer == signer:
                 group = GROUP_FDROID_SIGNED
 
@@ -1014,7 +1022,7 @@ def v1_sort_packages(packages, fdroid_signing_key_fingerprints):
     packages.sort(key=v1_sort_keys)
 
 
-def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fingerprints):
+def make_v0(apps, apks, repodir, repodict, requestsdict, signer_fingerprints):
     """Aka index.jar aka index.xml."""
     doc = Document()
 
@@ -1113,7 +1121,7 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
                 if name_from_apk is None:
                     name_from_apk = apk.get('name')
         for versionCode, apksforver in apksbyversion.items():
-            fdroid_signer = fdroid_signing_key_fingerprints.get(appid, {}).get('signer')
+            fdroid_signer = signer_fingerprints.get(appid, {}).get('signer')
             fdroid_signed_apk = None
             name_match_apk = None
             for x in apksforver:
@@ -1378,7 +1386,15 @@ def make_v0(apps, apks, repodir, repodict, requestsdict, fdroid_signing_key_fing
                         % repo_icon)
         os.makedirs(os.path.dirname(iconfilename), exist_ok=True)
         try:
+            import qrcode
+
             qrcode.make(common.config['repo_url']).save(iconfilename)
+        except ModuleNotFoundError as e:
+            raise ModuleNotFoundError(
+                _(
+                    'The "qrcode" Python package is not installed (e.g. apt-get install python3-qrcode)!'
+                )
+            ) from e
         except Exception:
             exampleicon = os.path.join(common.get_examples_dir(),
                                        common.default_config['repo_icon'])
@@ -1445,7 +1461,7 @@ def add_mirrors_to_repodict(repo_section, repodict):
                 )
             )
         with mirrors_yml.open() as fp:
-            mirrors_config = ruamel.yaml.YAML(typ='safe').load(fp)
+            mirrors_config = yaml.load(fp)
         if not isinstance(mirrors_config, list):
             msg = _('{path} is not list, but a {datatype}!')
             raise TypeError(
@@ -1499,6 +1515,7 @@ def add_mirrors_to_repodict(repo_section, repodict):
     repodict['mirrors'] = []
     canonical_url = repodict['address']
     found_primary = False
+    errors = 0
     for mirror in mirrors:
         if canonical_url == mirror['url']:
             found_primary = True
@@ -1507,9 +1524,19 @@ def add_mirrors_to_repodict(repo_section, repodict):
             for k in sorted(mirror.keys()):
                 sortedmirror[k] = mirror[k]
             repodict['mirrors'].insert(0, sortedmirror)
+        elif mirror.get('isPrimary'):
+            errors += 1
+            logging.error(
+                _('Mirror config for {url} contains "isPrimary" key!').format(
+                    url=mirror['url']
+                )
+            )
         else:
             repodict['mirrors'].append(mirror)
 
+    if errors:
+        raise FDroidException(_('"isPrimary" key should not be added to mirrors!'))
+
     if repodict['mirrors'] and not found_primary:
         repodict['mirrors'].insert(0, {'isPrimary': True, 'url': repodict['address']})
 
@@ -1624,6 +1651,8 @@ def download_repo_index_v1(url_str, etag=None, verify_fingerprint=True, timeout=
         - The new eTag as returned by the HTTP request
 
     """
+    from . import net
+
     url = urllib.parse.urlsplit(url_str)
 
     fingerprint = None
@@ -1675,6 +1704,8 @@ def download_repo_index_v2(url_str, etag=None, verify_fingerprint=True, timeout=
         - The new eTag as returned by the HTTP request
 
     """
+    from . import net
+
     etag  # etag is unused but needs to be there to keep the same API as the earlier functions.
 
     url = urllib.parse.urlsplit(url_str)
