@@ -24,9 +24,8 @@ import urllib.parse
 from argparse import ArgumentParser
 from pathlib import Path
 
-import ruamel.yaml
-
 from . import _, common, metadata, rewritemeta
+from fdroidserver._yaml import yaml
 
 config = None
 
@@ -213,6 +212,84 @@ regex_checks = {
     ],
 }
 
+# config keys that are currently ignored by lint, but could be supported.
+ignore_config_keys = (
+    'github_releases',
+    'java_paths',
+)
+
+bool_keys = (
+    'allow_disabled_algorithms',
+    'androidobservatory',
+    'build_server_always',
+    'deploy_process_logs',
+    'keep_when_not_allowed',
+    'make_current_version_link',
+    'nonstandardwebroot',
+    'per_app_repos',
+    'rclone',
+    'refresh_scanner',
+    's3cmd',
+    'scan_binary',
+    'sync_from_local_copy_dir',
+)
+
+check_config_keys = (
+    'ant',
+    'apk_signing_key_block_list',
+    'archive',
+    'archive_description',
+    'archive_icon',
+    'archive_name',
+    'archive_older',
+    'archive_url',
+    'archive_web_base_url',
+    'awsaccesskeyid',
+    'awsbucket',
+    'awssecretkey',
+    'binary_transparency_remote',
+    'cachedir',
+    'char_limits',
+    'current_version_name_source',
+    'git_mirror_size_limit',
+    'github_token',
+    'gpghome',
+    'gpgkey',
+    'gradle',
+    'identity_file',
+    'install_list',
+    'java_paths',
+    'keyaliases',
+    'keydname',
+    'keypass',
+    'keystore',
+    'keystorepass',
+    'lint_licenses',
+    'local_copy_dir',
+    'mirrors',
+    'mvn3',
+    'ndk_paths',
+    'path_to_custom_rclone_config',
+    'rclone_config',
+    'repo',
+    'repo_description',
+    'repo_icon',
+    'repo_keyalias',
+    'repo_maxage',
+    'repo_name',
+    'repo_pubkey',
+    'repo_url',
+    'repo_web_base_url',
+    'scanner_signature_sources',
+    'sdk_path',
+    'servergitmirrors',
+    'serverwebroot',
+    'smartcardoptions',
+    'sync_from_local_copy_dir',
+    'uninstall_list',
+    'virustotal_apikey',
+)
+
 locale_pattern = re.compile(r"[a-z]{2,3}(-([A-Z][a-zA-Z]+|\d+|[a-z]+))*")
 
 versioncode_check_pattern = re.compile(r"(\\d|\[(0-9|\\d)_?(a-fA-F)?])[+]")
@@ -501,11 +578,20 @@ def check_format(app):
 
 
 def check_license_tag(app):
-    """Ensure all license tags contain only valid/approved values."""
-    if config['lint_licenses'] is None:
-        return
-    if app.License not in config['lint_licenses']:
-        if config['lint_licenses'] == APPROVED_LICENSES:
+    """Ensure all license tags contain only valid/approved values.
+
+    It is possible to disable license checking by setting a null or empty value,
+    e.g. `lint_licenses: ` or `lint_licenses: []`
+
+    """
+    if 'lint_licenses' in config:
+        lint_licenses = config['lint_licenses']
+        if lint_licenses is None:
+            return
+    else:
+        lint_licenses = APPROVED_LICENSES
+    if app.License not in lint_licenses:
+        if lint_licenses == APPROVED_LICENSES:
             yield _(
                 'Unexpected license tag "{}"! Only use FSF or OSI '
                 'approved tags from https://spdx.org/license-list'
@@ -767,7 +853,7 @@ def lint_config(arg):
         passed = False
 
     with path.open() as fp:
-        data = ruamel.yaml.YAML(typ='safe').load(fp)
+        data = yaml.load(fp)
     common.config_type_check(arg, data)
 
     if path.name == mirrors_name:
@@ -791,6 +877,41 @@ def lint_config(arg):
                     msg += ' '
                     msg += _('Did you mean {code}?').format(code=', '.join(sorted(m)))
                 print(msg)
+    elif path.name == config_name and path.parent.name != 'config':
+        valid_keys = set(tuple(common.default_config) + bool_keys + check_config_keys)
+        for key in ignore_config_keys:
+            if key in valid_keys:
+                valid_keys.remove(key)
+        for key in data:
+            if key not in valid_keys:
+                passed = False
+                msg = _("ERROR: {key} not a valid key!").format(key=key)
+                m = difflib.get_close_matches(key.lower(), valid_keys, 2, 0.5)
+                if m:
+                    msg += ' '
+                    msg += _('Did you mean {code}?').format(code=', '.join(sorted(m)))
+                print(msg)
+                continue
+
+            if key in bool_keys:
+                t = bool
+            else:
+                t = type(common.default_config.get(key, ""))
+
+            show_error = False
+            if t is str:
+                if type(data[key]) not in (str, list, dict):
+                    passed = False
+                    show_error = True
+            elif type(data[key]) != t:
+                passed = False
+                show_error = True
+            if show_error:
+                print(
+                    _("ERROR: {key}'s value should be of type {t}!").format(
+                        key=key, t=t.__name__
+                    )
+                )
     elif path.name in (config_name, categories_name, antifeatures_name):
         for key in data:
             if path.name == config_name and key not in ('archive', 'repo'):
@@ -864,7 +985,7 @@ def main():
     paths = list()
     for arg in options.appid:
         if (
-            arg == 'config.yml'
+            arg == common.CONFIG_FILE
             or Path(arg).parent.name == 'config'
             or Path(arg).parent.parent.name == 'config'  # localized
         ):
