@@ -139,13 +139,13 @@ def sign_sig_key_fingerprint_list(jar_file):
         raise FDroidException("Failed to sign '{}'!".format(jar_file))
 
 
-def store_stats_fdroid_signing_key_fingerprints(appids, indent=None):
+def store_publish_signer_fingerprints(appids, indent=None):
     """Store list of all signing-key fingerprints for given appids to HD.
 
     This list will later on be needed by fdroid update.
     """
-    if not os.path.exists('stats'):
-        os.makedirs('stats')
+    if not os.path.exists('repo'):
+        os.makedirs('repo')
     data = OrderedDict()
     fps = read_fingerprints_from_keystore()
     for appid in sorted(appids):
@@ -153,9 +153,12 @@ def store_stats_fdroid_signing_key_fingerprints(appids, indent=None):
         if alias in fps:
             data[appid] = {'signer': fps[key_alias(appid)]}
 
-    jar_file = os.path.join('stats', 'publishsigkeys.jar')
+    jar_file = os.path.join('repo', 'signer-index.jar')
+    output = json.dumps(data, indent=indent)
     with zipfile.ZipFile(jar_file, 'w', zipfile.ZIP_DEFLATED) as jar:
-        jar.writestr('publishsigkeys.json', json.dumps(data, indent=indent))
+        jar.writestr('signer-index.json', output)
+    with open(os.path.join('repo', 'signer-index.json'), 'w') as fp:
+        fp.write(output)
     sign_sig_key_fingerprint_list(jar_file)
 
 
@@ -344,7 +347,6 @@ def main():
         glob.glob(os.path.join(unsigned_dir, '*.apk'))
         + glob.glob(os.path.join(unsigned_dir, '*.zip'))
     ):
-
         appid, vercode = common.publishednameinfo(apkfile)
         apkfilename = os.path.basename(apkfile)
         if vercodes and appid not in vercodes:
@@ -368,7 +370,6 @@ def main():
             if b.get("versionCode") == vercode:
                 build = b
         if app.Binaries or (build and build.binary):
-
             # It's an app where we build from source, and verify the apk
             # contents against a developer's binary, and then publish their
             # version if everything checks out.
@@ -379,15 +380,21 @@ def main():
             srcapk = srcapk.replace(unsigned_dir, binaries_dir)
 
             if not os.path.isfile(srcapk):
-                logging.error("...reference binary missing - publish skipped: "
-                              "'{refpath}'".format(refpath=srcapk))
+                logging.error(
+                    "...reference binary missing - publish skipped: '{refpath}'".format(
+                        refpath=srcapk
+                    )
+                )
                 failed += 1
             else:
                 # Compare our unsigned one with the downloaded one...
                 compare_result = common.verify_apks(srcapk, apkfile, tmp_dir)
                 if compare_result:
-                    logging.error("...verification failed - publish skipped : "
-                                  "{result}".format(result=compare_result))
+                    logging.error(
+                        "...verification failed - publish skipped : {result}".format(
+                            result=compare_result
+                        )
+                    )
                     failed += 1
                 else:
                     # Success! So move the downloaded file to the repo, and remove
@@ -399,7 +406,6 @@ def main():
                     logging.info('Published ' + apkfilename)
 
         elif apkfile.endswith('.zip'):
-
             # OTA ZIPs built by fdroid do not need to be signed by jarsigner,
             # just to be moved into place in the repo
             shutil.move(apkfile, os.path.join(output_dir, apkfilename))
@@ -407,7 +413,6 @@ def main():
             logging.info('Published ' + apkfilename)
 
         else:
-
             # It's a 'normal' app, i.e. we sign and publish it...
             skipsigning = False
 
@@ -446,10 +451,11 @@ def main():
 
                 signed_apk_path = os.path.join(output_dir, apkfilename)
                 if os.path.exists(signed_apk_path):
-                    raise BuildException("Refusing to sign '{0}' file exists in both "
-                                         "{1} and {2} folder.".format(apkfilename,
-                                                                      unsigned_dir,
-                                                                      output_dir))
+                    raise BuildException(
+                        _(
+                            "Refusing to sign '{path}', file exists in both {dir1} and {dir2} folder."
+                        ).format(path=apkfilename, dir1=unsigned_dir, dir2=output_dir)
+                    )
 
                 # Sign the application...
                 common.sign_apk(apkfile, signed_apk_path, keyalias)
@@ -460,7 +466,7 @@ def main():
                 publish_source_tarball(apkfilename, unsigned_dir, output_dir)
                 logging.info('Published ' + apkfilename)
 
-    store_stats_fdroid_signing_key_fingerprints(allapps.keys())
+    store_publish_signer_fingerprints(allapps.keys())
     status_update_json(generated_keys, signed_apks)
     logging.info('published list signing-key fingerprints')
 
