@@ -2,6 +2,7 @@
 
 import os
 import shutil
+import sys
 import tempfile
 import textwrap
 import unittest
@@ -9,7 +10,7 @@ import yaml
 from pathlib import Path
 from unittest import mock
 
-from .testcommon import TmpCwd, mkdtemp
+from .shared_test_code import TmpCwd, mkdtemp
 
 import fdroidserver.build
 import fdroidserver.common
@@ -48,6 +49,7 @@ class BuildTest(unittest.TestCase):
         os.makedirs(os.path.join(d, 'platform-tools'), exist_ok=True)
         os.makedirs(os.path.join(d, 'tools'), exist_ok=True)
 
+    @unittest.skipIf(sys.byteorder == 'big', "androguard is not ported to big-endian")
     def test_get_apk_metadata(self):
         config = dict()
         fdroidserver.common.fill_config_defaults(config)
@@ -593,9 +595,9 @@ class BuildTest(unittest.TestCase):
         os.chdir(self.testdir)
         sdk_path = os.path.join(self.testdir, 'android-sdk')
         self.create_fake_android_home(sdk_path)
-        with open('config.yml', 'w') as fp:
+        with open(fdroidserver.common.CONFIG_FILE, 'w') as fp:
             yaml.dump({'sdk_path': sdk_path, 'keep_when_not_allowed': True}, fp)
-        os.chmod('config.yml', 0o600)
+        os.chmod(fdroidserver.common.CONFIG_FILE, 0o600)
         fdroidserver.build.config = fdroidserver.common.read_config()
 
         os.mkdir('metadata')
