@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 
+import logging
 import os
 import pathlib
 import re
@@ -22,13 +23,29 @@ import yaml
 
 import fdroidserver.build
 import fdroidserver.common
+import fdroidserver.exception
 import fdroidserver.metadata
 import fdroidserver.scanner
-from .testcommon import TmpCwd, mkdtemp, mock_open_to_str
+from .shared_test_code import TmpCwd, mkdtemp, mock_open_to_str
 
 basedir = pathlib.Path(__file__).parent
 
 
+def _dexdump_found():
+    """Find if dexdump is available in the PATH or in an Android SDK install.
+
+    This must be run after common.config is setup.
+
+    """
+    try:
+        dexdump = fdroidserver.common.find_sdk_tools_cmd("dexdump")
+        logging.debug('Found dexdump: %s', dexdump)
+        return dexdump is not None
+    except fdroidserver.exception.FDroidException:
+        pass
+    return False
+
+
 # Always use built-in default rules so changes in downloaded rules don't break tests.
 @mock.patch(
     'fdroidserver.scanner.SUSSDataController.load',
@@ -66,8 +83,13 @@ class ScannerTest(unittest.TestCase):
         }
         for d in (basedir / 'source-files').iterdir():
             build = fdroidserver.metadata.Build()
-            fatal_problems = fdroidserver.scanner.scan_source(d, build)
             should = projects.get(d.name, 0)
+            if should > 0:
+                with self.assertLogs(level=logging.ERROR):
+                    fatal_problems = fdroidserver.scanner.scan_source(d, build)
+            else:
+                with self.assertNoLogs():
+                    fatal_problems = fdroidserver.scanner.scan_source(d, build)
             self.assertEqual(
                 should, fatal_problems, f'{d} should have {should} errors!'
             )
@@ -174,7 +196,8 @@ class ScannerTest(unittest.TestCase):
             """
                 )
             )
-        count = fdroidserver.scanner.scan_source(self.testdir)
+        with self.assertLogs(level=logging.ERROR):
+            count = fdroidserver.scanner.scan_source(self.testdir)
         self.assertEqual(2, count, 'there should be this many errors')
 
     def test_scan_source_file_types(self):
@@ -226,9 +249,10 @@ class ScannerTest(unittest.TestCase):
         # run scanner as if from `fdroid build`
         os.chdir(self.testdir)
         json_per_build = fdroidserver.scanner.MessageStore()
-        count = fdroidserver.scanner.scan_source(
-            build_dir, json_per_build=json_per_build
-        )
+        with self.assertLogs(level=logging.ERROR):
+            count = fdroidserver.scanner.scan_source(
+                build_dir, json_per_build=json_per_build
+            )
         self.assertEqual(6, count, 'there should be this many errors')
         os.chdir(build_dir)
 
@@ -391,6 +415,8 @@ class ScannerTest(unittest.TestCase):
         config = dict()
         fdroidserver.common.config = config
         fdroidserver.common.fill_config_defaults(config)
+        if not _dexdump_found():
+            self.skipTest('Some Debian arches lack dexdump')
         for f in (
             'apk.embedded_1.apk',
             'bad-unicode-πÇÇ现代通用字-български-عربي1.apk',
@@ -436,7 +462,6 @@ class ScannerTest(unittest.TestCase):
         config = dict()
         fdroidserver.common.config = config
         fdroidserver.common.fill_config_defaults(config)
-        print('basedir')
         for f in (
             'Norway_bouvet_europe_2.obf.zip',
             'repo/fake.ota.update_1234.zip',
@@ -447,14 +472,13 @@ class ScannerTest(unittest.TestCase):
                 'should return not results for ' + f,
             )
 
-    @unittest.skipIf(
-        sys.hexversion < 0x03090000, 'Python < 3.9 has a limited zipfile.is_zipfile()'
-    )
     def test_get_embedded_classes_secret_apk(self):
         """Try to hide an APK+DEX in an APK and see if we can find it"""
         config = dict()
         fdroidserver.common.config = config
         fdroidserver.common.fill_config_defaults(config)
+        if not _dexdump_found():
+            self.skipTest('Some Debian arches lack dexdump')
         apk = 'urzip.apk'
         mapzip = 'Norway_bouvet_europe_2.obf.zip'
         secretfile = os.path.join(
@@ -494,6 +518,9 @@ class Test_scan_binary(unittest.TestCase):
         fdroidserver.common.config = config
         fdroidserver.common.options = mock.Mock()
 
+        if not _dexdump_found():
+            self.skipTest('Some Debian arches lack dexdump')
+
         fdroidserver.scanner._SCANNER_TOOL = mock.Mock()
         fdroidserver.scanner._SCANNER_TOOL.regexs = {}
         fdroidserver.scanner._SCANNER_TOOL.regexs['err_code_signatures'] = {
@@ -505,9 +532,11 @@ class Test_scan_binary(unittest.TestCase):
 
     def test_code_signature_match(self):
         apkfile = os.path.join(basedir, 'no_targetsdk_minsdk1_unsigned.apk')
+        with self.assertLogs(level=logging.CRITICAL):
+            problems = fdroidserver.scanner.scan_binary(apkfile)
         self.assertEqual(
             1,
-            fdroidserver.scanner.scan_binary(apkfile),
+            problems,
             "Did not find expected code signature '{}' in binary '{}'".format(
                 fdroidserver.scanner._SCANNER_TOOL.regexs[
                     'err_code_signatures'
@@ -516,12 +545,6 @@ class Test_scan_binary(unittest.TestCase):
             ),
         )
 
-    @unittest.skipIf(
-        sys.version_info < (3, 9),
-        "Our implementation for traversing zip files will silently fail to work"
-        "on older python versions, also see: "
-        "https://gitlab.com/fdroid/fdroidserver/-/merge_requests/1110#note_932026766",
-    )
     def test_bottom_level_embedded_apk_code_signature(self):
         apkfile = os.path.join(basedir, 'apk.embedded_1.apk')
         fdroidserver.scanner._SCANNER_TOOL.regexs['err_code_signatures'] = {
@@ -531,9 +554,11 @@ class Test_scan_binary(unittest.TestCase):
             )
         }
 
+        with self.assertLogs(level=logging.CRITICAL):
+            problems = fdroidserver.scanner.scan_binary(apkfile)
         self.assertEqual(
             1,
-            fdroidserver.scanner.scan_binary(apkfile),
+            problems,
             "Did not find expected code signature '{}' in binary '{}'".format(
                 fdroidserver.scanner._SCANNER_TOOL.regexs[
                     'err_code_signatures'
@@ -549,9 +574,11 @@ class Test_scan_binary(unittest.TestCase):
                 r'.*org/fdroid/ci/BuildConfig', re.IGNORECASE | re.UNICODE
             )
         }
+        with self.assertLogs(level=logging.CRITICAL):
+            problems = fdroidserver.scanner.scan_binary(apkfile)
         self.assertEqual(
             1,
-            fdroidserver.scanner.scan_binary(apkfile),
+            problems,
             "Did not find expected code signature '{}' in binary '{}'".format(
                 fdroidserver.scanner._SCANNER_TOOL.regexs[
                     'err_code_signatures'
@@ -800,7 +827,7 @@ class Test_ScannerTool(unittest.TestCase):
 
     def test_refresh_from_config(self):
         os.chdir(self.testdir)
-        pathlib.Path('config.yml').write_text('refresh_scanner: true')
+        fdroidserver.common.write_config_file('refresh_scanner: true\n')
         with mock.patch('fdroidserver.scanner.ScannerTool.refresh') as refresh:
             fdroidserver.scanner.ScannerTool()
             refresh.assert_called_once()
@@ -809,7 +836,7 @@ class Test_ScannerTool(unittest.TestCase):
         fdroidserver.common.options = mock.Mock()
         fdroidserver.common.options.refresh_scanner = True
         os.chdir(self.testdir)
-        pathlib.Path('config.yml').write_text('refresh_scanner: false')
+        fdroidserver.common.write_config_file('refresh_scanner: false\n')
         with mock.patch('fdroidserver.scanner.ScannerTool.refresh') as refresh:
             fdroidserver.scanner.ScannerTool()
             refresh.assert_called_once()
