@@ -11,6 +11,7 @@ import random
 import shutil
 import string
 import subprocess
+import sys
 import unittest
 import yaml
 import zipfile
@@ -48,7 +49,7 @@ import fdroidserver.metadata
 import fdroidserver.update
 from fdroidserver.common import CATEGORIES_CONFIG_NAME
 from fdroidserver.looseversion import LooseVersion
-from .testcommon import TmpCwd, mkdtemp
+from .shared_test_code import TmpCwd, mkdtemp
 from PIL import PngImagePlugin
 
 
@@ -68,6 +69,7 @@ class Options:
     verbose = False
 
 
+@unittest.skipIf(sys.byteorder == 'big', 'androguard is not ported to big-endian')
 class UpdateTest(unittest.TestCase):
     '''fdroid update'''
 
@@ -603,7 +605,7 @@ class UpdateTest(unittest.TestCase):
         apps = fdroidserver.metadata.read_metadata()
         knownapks = fdroidserver.common.KnownApks()
         apks, cachechanged = fdroidserver.update.process_apks({}, 'repo', knownapks, False)
-        self.assertEqual(len(apks), 17)
+        self.assertEqual(len(apks), 18)
         apk = apks[1]
         self.assertEqual(apk['packageName'], 'com.politedroid')
         self.assertEqual(apk['versionCode'], 3)
@@ -666,7 +668,7 @@ class UpdateTest(unittest.TestCase):
 
         fdroidserver.update.options.clean = False
         read_from_json = fdroidserver.update.get_cache()
-        self.assertEqual(19, len(read_from_json))
+        self.assertEqual(20, len(read_from_json))
         for f in glob.glob('repo/*.apk'):
             self.assertTrue(os.path.basename(f) in read_from_json)
 
@@ -1239,7 +1241,7 @@ class UpdateTest(unittest.TestCase):
         knownapks = fdroidserver.common.KnownApks()
         apks, cachechanged = fdroidserver.update.process_apks({}, 'repo', knownapks, False)
         fdroidserver.update.translate_per_build_anti_features(apps, apks)
-        self.assertEqual(len(apks), 17)
+        self.assertEqual(len(apks), 18)
         foundtest = False
         for apk in apks:
             if apk['packageName'] == 'com.politedroid' and apk['versionCode'] == 3:
@@ -1787,7 +1789,9 @@ class UpdateTest(unittest.TestCase):
     def test_categories_txt_is_removed_by_delete_unknown(self):
         """categories.txt used to be a part of this system, now its nothing."""
         os.chdir(self.testdir)
-        Path('config.yml').write_text('repo_pubkey: ffffffffffffffffffffffffffffffffffffffff')
+        fdroidserver.common.write_config_file(
+            'repo_pubkey: ffffffffffffffffffffffffffffffffffffffff\n'
+        )
 
         categories_txt = Path('repo/categories.txt')
         categories_txt.parent.mkdir()
@@ -1803,8 +1807,8 @@ class UpdateTest(unittest.TestCase):
         os.chdir(self.testdir)
         os.mkdir('metadata')
         os.mkdir('repo')
-        Path('config.yml').write_text(
-            'repo_pubkey: ffffffffffffffffffffffffffffffffffffffff'
+        fdroidserver.common.write_config_file(
+            'repo_pubkey: ffffffffffffffffffffffffffffffffffffffff\n'
         )
 
         testapk = os.path.join('repo', 'com.politedroid_6.apk')
@@ -1822,8 +1826,8 @@ class UpdateTest(unittest.TestCase):
         os.chdir(self.testdir)
         os.mkdir('metadata')
         os.mkdir('repo')
-        Path('config.yml').write_text(
-            'repo_pubkey: ffffffffffffffffffffffffffffffffffffffff'
+        fdroidserver.common.write_config_file(
+            'repo_pubkey: ffffffffffffffffffffffffffffffffffffffff\n'
         )
 
         testapk = os.path.join('repo', 'com.politedroid_6.apk')
@@ -1844,8 +1848,8 @@ class UpdateTest(unittest.TestCase):
         os.chdir(self.testdir)
         os.mkdir('metadata')
         os.mkdir('repo')
-        Path('config.yml').write_text(
-            'repo_pubkey: ffffffffffffffffffffffffffffffffffffffff'
+        fdroidserver.common.write_config_file(
+            'repo_pubkey: ffffffffffffffffffffffffffffffffffffffff\n'
         )
 
         testapk = os.path.join('repo', 'com.politedroid_6.apk')
@@ -1871,8 +1875,8 @@ class UpdateTest(unittest.TestCase):
         Path('config/categories.yml').write_text('System: {name: System Apps}')
         os.mkdir('metadata')
         os.mkdir('repo')
-        Path('config.yml').write_text(
-            'repo_pubkey: ffffffffffffffffffffffffffffffffffffffff'
+        fdroidserver.common.write_config_file(
+            'repo_pubkey: ffffffffffffffffffffffffffffffffffffffff\n'
         )
 
         testapk = os.path.join('repo', 'com.politedroid_6.apk')
@@ -1901,8 +1905,8 @@ class UpdateTest(unittest.TestCase):
         Path('config/categories.yml').write_text('System: {name: S}\nTime: {name: T}\n')
         os.mkdir('metadata')
         os.mkdir('repo')
-        Path('config.yml').write_text(
-            'repo_pubkey: ffffffffffffffffffffffffffffffffffffffff'
+        fdroidserver.common.write_config_file(
+            'repo_pubkey: ffffffffffffffffffffffffffffffffffffffff\n'
         )
 
         testapk = os.path.join('repo', 'com.politedroid_6.apk')
